NAME = "vents"
VERSION = "0.6.2"
DESC = "Open source connections, integrations, alerting, and notification library."
URL = "https://github.com/mmourafiq/vents"
AUTHOR = "Mourad Mourafiq"
EMAIL = "mourad.mourafiq@gmail.com"
LICENSE = "Apache 2.0"
